"""Provides an example how to use mid level layer"""

import sys
import asyncio

from science_mode_4 import DeviceP24
from science_mode_4 import MidLevelChannelConfiguration
from science_mode_4 import ChannelPoint
from science_mode_4 import SerialPortConnection
from examples.utils.example_utils import ExampleUtils, KeyboardInputThread


async def main() -> int:
    """Main function"""

    # some points
    p1: ChannelPoint = ChannelPoint(200, 20)
    p2: ChannelPoint = ChannelPoint(100, 0)
    p3: ChannelPoint = ChannelPoint(200, -20)
    # channel configuration
    # we want to ignore first and last two channels (just for demonstration purpose how to handle unused channels)
    # we need to pad list with None to achieve correct indices
    # [None, None, ChannelConfig, ChannelConfig, ChannelConfig, ChannelConfig]
    channel_config = [MidLevelChannelConfiguration(False, 3, 20, [p1, p2, p3]) for x in range(4)]
    channel_config.insert(0, None)
    channel_config.insert(0, None)

    # keyboard is our trigger to end program
    def input_callback(input_value: str) -> bool:
        """Callback call from keyboard input thread"""
        # print(f"Input value {input_value}")

        if input_value == "q":
            # end keyboard input thread
            return True
        if  "1" <= input_value <= "8":
            index = int(input_value) - 1
            # check if index is in range of channel_config
            if 0 <= index < len(channel_config):
                cc = channel_config[index]
                # check if index contains a ChannelConfiguration object
                if cc is not None:
                    # toggle active
                    cc.is_active = not cc.is_active
                    asyncio.run(mid_level.update(channel_config))
                else:
                    print("Channel config is None")
            else:
                print("Invalid channel config index")

            return False


        print("Invalid command")
        return False

    print("Usage: press 1-8 to toggle channel, press q to quit")
    print("Only channels 3-6 have a channel configuration (see comments where configuration is created)")
    # create keyboard input thread for non blocking console input
    keyboard_input_thread = KeyboardInputThread(input_callback)

    # get comport from command line argument
    com_port = ExampleUtils.get_comport_from_commandline_argument()
    # create serial port connection
    connection = SerialPortConnection(com_port)
    # open connection, now we can read and write data
    connection.open()

    # create science mode device
    device = DeviceP24(connection)
    # call initialize to get basic information (serial, versions) and stop any active stimulation/measurement
    # to have a defined state
    await device.initialize()

    # get mid level layer to call mid level commands
    mid_level = device.get_layer_mid_level()
    # call init mid level, we want to stop on all stimulation errors
    await mid_level.init(True)
    # set stimulation pattern, P24 device will now stimulate according this pattern
    await mid_level.update(channel_config)

    while keyboard_input_thread.is_alive():
        # we have to call get_current_data() every 1.5s to keep stimulation ongoing
        update = await mid_level.get_current_data() # pylint:disable=unused-variable
        # print(update)

        await asyncio.sleep(1)

    # call stop mid level
    await mid_level.stop()

    # close serial port connection
    connection.close()
    return 0


if __name__ == "__main__":
    res = asyncio.run(main())
    sys.exit(res)



"""Test program how to use library without installing the library,
DO NOT USE THIS FILE, USE EXAMPLES INSTEAD"""

import sys
import asyncio

from science_mode_4.device_i24 import DeviceI24
from science_mode_4.dyscom.dyscom_types import DyscomFilterType, DyscomInitFlag, DyscomInitParams, DyscomPowerModulePowerType,\
    DyscomPowerModuleType, DyscomSignalType, DyscomSysType
from science_mode_4.utils.serial_port_connection import SerialPortConnection


async def main() -> int:
    """Main function"""

    # create serial port connection
    connection = SerialPortConnection(SerialPortConnection.list_science_mode_device_ports()[0].device)
    # open connection, now we can read and write data
    connection.open()

    # create science mode device
    device = DeviceI24(connection)
    # call initialize to get basic information (serial, versions) and stop any active stimulation/measurement
    # to have a defined state
    await device.initialize()


    # get dyscom layer to call low level commands
    dyscom = device.get_layer_dyscom()
    device_id = await dyscom.get_device_id()

    ####
    calibration_filename = f"rehaingest_{device_id}.cal"
    calibration_content = await dyscom.get_file_content(calibration_filename)
    print(f"Calibration content length: {len(calibration_content)}")
    calculated_checksum = (calibration_content[0] << 8) | calibration_content[1]
    print(f"Calculated calibration content checksum: {calculated_checksum}")

    ####
    file_info = await dyscom.get_file_info(calibration_filename)
    print(f"Calibration file info checksum: {file_info.checksum}")

    s_y_s = await dyscom.sys(DyscomSysType.DEVICE_STORAGE)
    print(f"Sys {s_y_s.state.name}")

    # call enable measurement power module and memory card for measurement
    await dyscom.power_module(DyscomPowerModuleType.MEASUREMENT, DyscomPowerModulePowerType.SWITCH_ON)
    await dyscom.power_module(DyscomPowerModuleType.MEMORY_CARD, DyscomPowerModulePowerType.SWITCH_ON)
    # call init with lowest sample rate (because of performance issues with plotting values)
    init_params = DyscomInitParams()
    init_params.signal_type = [DyscomSignalType.BI, DyscomSignalType.EMG_1]
    init_params.filter = DyscomFilterType.PREDEFINED_FILTER_1
    # we want no live data and write all data to memory card
    init_params.flags = [DyscomInitFlag.ENABLE_SD_STORAGE_MODE]
    init_result = await dyscom.init(init_params)

    # start dyscom measurement
    await dyscom.start()

    # wait 10s to have some measurement data
    await asyncio.sleep(10)

    # stop measurement
    await dyscom.stop()
    # turn power module off
    await dyscom.power_module(DyscomPowerModuleType.MEASUREMENT, DyscomPowerModulePowerType.SWITCH_OFF)

    # get all meas data
    measurement = await dyscom.get_meas_file_content(init_result.measurement_file_id)
    print(f"Sample rate: {measurement[0].name}")
    for key, value in measurement[1].items():
        print(f"Signal type: {key.name}, sample count: {len(value)}")

    # turn memory card off
    await dyscom.power_module(DyscomPowerModuleType.MEMORY_CARD, DyscomPowerModulePowerType.SWITCH_OFF)
    # close serial port connection
    connection.close()

    return 0


if __name__ == "__main__":
    res = asyncio.run(main())
    sys.exit(res)



"""Provides an example how to use dyscom level layer to read stored data from device"""

import asyncio

from science_mode_4 import DeviceI24
from science_mode_4 import SerialPortConnection
from science_mode_4.dyscom.dyscom_types import DyscomFilterType, DyscomInitFlag, DyscomInitParams, DyscomPowerModulePowerType,\
    DyscomPowerModuleType, DyscomSignalType
from examples.utils.example_utils import ExampleUtils


async def main() -> int:
    """Main function"""

    # get comport from command line argument
    com_port = ExampleUtils.get_comport_from_commandline_argument()
    # create serial port connection
    connection = SerialPortConnection(com_port)
    # open connection, now we can read and write data
    connection.open()

    # create science mode device
    device = DeviceI24(connection)
    # call initialize to get basic information (serial, versions) and stop any active stimulation/measurement
    # to have a defined state
    await device.initialize()

    # get dyscom layer to call low level commands
    dyscom = device.get_layer_dyscom()

    # call enable measurement power module and memory card for measurement
    await dyscom.power_module(DyscomPowerModuleType.MEASUREMENT, DyscomPowerModulePowerType.SWITCH_ON)
    await dyscom.power_module(DyscomPowerModuleType.MEMORY_CARD, DyscomPowerModulePowerType.SWITCH_ON)
    # call init with 1k sample rate
    init_params = DyscomInitParams()
    init_params.signal_type = [DyscomSignalType.BI, DyscomSignalType.EMG_1]
    init_params.filter = DyscomFilterType.PREDEFINED_FILTER_1
    # we want no live data and write all data to memory card
    init_params.flags = [DyscomInitFlag.ENABLE_SD_STORAGE_MODE]
    init_result = await dyscom.init(init_params)

    # start dyscom measurement
    await dyscom.start()

    # wait 10s to have some measurement data
    await asyncio.sleep(10)

    # stop measurement
    await dyscom.stop()
    # turn power module off
    await dyscom.power_module(DyscomPowerModuleType.MEASUREMENT, DyscomPowerModulePowerType.SWITCH_OFF)

    # get all meas data
    measurement = await dyscom.get_meas_file_content(init_result.measurement_file_id)
    print(f"Sample rate: {measurement[0].name}")
    for key, value in measurement[1].items():
        print(f"Signal type: {key.name}, sample count: {len(value)}")

    # turn memory card off
    await dyscom.power_module(DyscomPowerModuleType.MEMORY_CARD, DyscomPowerModulePowerType.SWITCH_OFF)
    # close serial port connection
    connection.close()

    return 0


if __name__ == "__main__":
    asyncio.run(main())
